/*
    This file is part of KitchenSync.

    Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>
    Copyright (c) 2004 Maciek Borowka <maciek@borowka.net>    

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include "mlkonnector.h"

#include "mlkonnectorconfig.h"

#include "mlhttpserver.h"
#include "mlsessionhandler.h"

#include "addressbookdatabase.h"
#include "calendardatabase.h"


#include <qtimer.h>
#include <calendarsyncee.h>
#include <konnectorinfo.h>
#include <kconfig.h>
#include <kgenericfactory.h>

#include <libkcal/vcalformat.h>
#include <libkcal/icalformat.h>

#include <kabc/vcardconverter.h> 

using namespace KSync;

extern "C"
{
	void *init_libsyncmlkonnector()
	{
		return new KRES::PluginFactory<MLKonnector,MLKonnectorConfig>();
	}
}


class MLKonnector::Private
{
  public:
    Private() : sessionHandler(NULL) {}

    SMLSessionHandler *sessionHandler;
};

QStringList MLKonnector::supportedFilterTypes() const
{
	QStringList types;
	//types << "addressbook" << "calendar";
	types << "calendar";
	return types;
}


MLKonnector::MLKonnector( const KConfig *config )
    : Konnector( config ), mConfigWidget( 0 )
{
	
	#ifdef DEBUG_MEM
	kdDebug() << "DEBUG_MEM - MLKonnector: created." << endl;
	#endif
	
	d = new Private;

	if ( config ) {
	}

/* 	XXX TODO: Make calendar database work. But for now, let's implement all the features only on addressdatabase
	mCalendarDatabase = new KSyncML::CalendarDatabase();
	QObject::connect(mCalendarDatabase, SIGNAL(synceesRead(KSyncML::SyncMLDatabase* )),
			this, SLOT(slotSynceesRead(KSyncML::SyncMLDatabase* )));
*/

}

MLKonnector::~MLKonnector()
{
	//delete mAddressBookDatabase;
	//delete mCalendarDatabase;
	delete d;
	
	#ifdef DEBUG_MEM
	kdDebug() << "DEBUG_MEM - MLKonnector: destroyed." << endl;
	#endif
	
}

void MLKonnector::writeConfig( KConfig *config )
{
  Konnector::writeConfig( config );
}
/*
KSync::Kapabilities MLKonnector::capabilities()
{
  KSync::Kapabilities caps;

  caps.setSupportMetaSyncing( false ); // we can meta sync
  caps.setSupportsPushSync( false ); // we can initialize the sync from here
  caps.setNeedsConnection( false ); // we need to have pppd running
  caps.setSupportsListDir( false ); // we will support that once there is API for it...
  caps.setNeedsIPs( true ); // we need the IP
  caps.setNeedsSrcIP( true ); // we do bind
  caps.setNeedsDestIP( false ); // we need to know where to connect
  caps.setAutoHandle( false ); // we currently do not support auto handling
  caps.setNeedAuthentication( false ); // HennevL says we do not need that
  caps.setNeedsModelName( false ); // we need a name for our meta path!

  return caps;
}
*/
KSync::KonnectorInfo MLKonnector::info() const
{
	return KonnectorInfo( i18n("cML Konnector"),
						  QIconSet(),
						  "agenda", // icon name
						  false );
}

/*
void MLKonnector::setCapabilities( const KSync::Kapabilities& )
{
}
*/
void MLKonnector::download( const QString& )
{
//  error( StdError::downloadNotSupported() );
}

bool MLKonnector::connectDevice()
{
	kdDebug(5201) << " starting the webserver ('communication layer'). " << endl;
	
	MLHttpServer * server = new MLHttpServer(this);
	QObject::connect(server, SIGNAL(newSession(SMLSessionHandler *)), 
			this, SLOT(slotNewSession(SMLSessionHandler *)));
	
	if (server->initialize())
	{
		return true;
	}
	else
	{
		delete server;
		return false;
	}
}


void MLKonnector::slotNewSession(SMLSessionHandler * sessionHandler)
{
	kdDebug(5201) << "MLKonnector: New session." << endl;
	d->sessionHandler = sessionHandler;
	
	QObject::connect(d->sessionHandler, SIGNAL(databaseDataRead(KSyncML::SyncMLDatabase* )),
					 this, SLOT(slotSynceesRead(KSyncML::SyncMLDatabase* )));

	QObject::connect(d->sessionHandler, SIGNAL(databaseDataWritten(KSyncML::SyncMLDatabase* )),
					 this, SLOT(slotSynceesWritten(KSyncML::SyncMLDatabase* )));
					
	QObject::connect(d->sessionHandler, SIGNAL(sessionFinished()),
					 this, SLOT(slotConversationFinished()));
	
	
	// XXX TODO: We need a way to correctly map the databases
	// to the clients in order to be able to support more than one client
	// That means that we need a way to pass the remote URI from the session handler
	// to all the databases.
	QString remoteDeviceID = "0000";
	
	KSyncML::AddressBookDatabase *adrDB = new KSyncML::AddressBookDatabase();
	adrDB->initializeDatabase(remoteDeviceID);
	
	KSyncML::CalendarDatabase *calDB = new KSyncML::CalendarDatabase();
	calDB->initializeDatabase(remoteDeviceID);
	
	
	d->sessionHandler->addDatabase(adrDB);
	d->sessionHandler->addDatabase(calDB);
	
	//
	// This is stupid.
	databasesToRead = 1;
	databasesToWrite = 1;
}

void MLKonnector::doReadSyncees()
{
	kdDebug(5201) << " reading syncees." << endl;
}

bool MLKonnector::readSyncees()
{
	doReadSyncees();
	return true;
}


void MLKonnector::emitSynceesRead()
{
	kdDebug() << "MLKonnector: Emit syncees read" << endl;
	emit synceesRead(this);
}


void MLKonnector::slotSynceesRead(KSyncML::SyncMLDatabase * base)
{
	if (databasesToRead)
	{
		kdDebug(5201) << "MLKonnector: database '" << base->getURI().getAbsoluteLocURI().c_str()
				<< "' read its data." << endl;
		
		if (base->shouldBeSynchronized())
		{
			--databasesToRead;
		
			KSync::Syncee * syncee = base->getSyncee();
			if (syncee->isValid())
				kdDebug(5201) << "MLKonnector: Returned valid syncee : " << syncee->identifier() << endl;
			else
				kdWarning(5201) << "MLKonnector: Returned INVALID syncee : " << syncee->identifier() << endl;
			
			mSyncees.append(base->getSyncee());
			
			if (databasesToRead == 0)
			{
				//kdDebug() << "MLKonnector: Syncees read. Fire the signal to inform the rest of the world." << endl;
				kdDebug() << "MLKonnector: Deferring emit synceesRead" << endl;
				//dumpSyncees();
				QTimer::singleShot(1, this, SLOT(emitSynceesRead()));
				//emitSynceesRead();
			}
		}
		else
		{
			kdDebug(5201) << "MLKonnector: Bogus! Should NOT be synchronized." << endl;
		}
	}
	else
		kdWarning(5201) << "MLKonnector: BUG! Base " << base->getURI().getAbsoluteLocURI().c_str()
		<< " wants us to read the data, but we have already read everything!" << endl;
	
}

void MLKonnector::doWriteSyncees()
{
	kdDebug(5201) << " writing syncees." << endl;
	
	d->sessionHandler->writeDatabases();
	
}

bool MLKonnector::writeSyncees()
{
	kdDebug() << "MLKonnector: Deferring write syncees" << endl;

	QTimer::singleShot(1, this, SLOT(doWriteSyncees()));
	
	return true;
}

void MLKonnector::slotSynceesWritten(KSyncML::SyncMLDatabase * base)
{
	if (databasesToWrite)
	{
		kdDebug(5201) << "MLKonnector: database '" << base->getURI().getAbsoluteLocURI().c_str()
				 << "' written its data." << endl;
		
		--databasesToWrite;
	}
	else
		kdWarning(5201) << "MLKonnector: BUG! Base " << base->getURI().getAbsoluteLocURI().c_str()
			<< " has written something but we didn't want it to do so" << endl;
	
}

void MLKonnector::slotConversationFinished()
{
	if (databasesToWrite != 0)
	{
		kdWarning(5201) << "MLKonnector: Conversation finished and databasesToWrite is " 
			<< databasesToWrite << endl;
		
	}
	
	kdDebug() << "MLKonnector: Deferring emit synceesWritten" << endl;
	QTimer::singleShot(1, this, SLOT(emitSynceesWritten()));
}


void MLKonnector::emitSynceesWritten()
{
	kdDebug() << "MLKonnector: Emit syncees written" << endl;
	emit synceesWritten(this);
}



bool MLKonnector::disconnectDevice()
{
	kdDebug(5201) << " NOT disconnecting. " << endl;
	
	return true;
}



#include "mlkonnector.moc"
