
#ifndef __SESSION_HANDLER_H
#define __SESSION_HANDLER_H

#include <qobject.h>

#include "syncml.h"
#include "mlsession.h"

#include "syncmldevinf.h"
#include "syncmldatabase.h"


class MLHttpServer;

//XXX TODO Merge SMLSessionHandler and SyncMLSession 
class SMLSessionHandler: public QObject
{
Q_OBJECT

private:
  SMLURI URI;
  SMLURI remoteURI;

  SMLResponsePacket *response;
  SMLDevInf *DevInf;
  SMLDevInf *remoteDevInf;

  SMLCmdRef CmdNoDevInf;
  SMLCmdMapType pending;
  KSyncML::SyncMLDatabaseMapType databaseMap;

  SMLVersion syncmlVersion;

  MLHttpServer * communication;
  //if true, the handler will send the response to the other side immediately after
  //receiving the "slotSessionFinished" from the session. It happens twice in the session,
  //at the beginning and at the end.
  bool automaticResponse;
  
  bool everythingFinishedFlag;


signals:
	void databaseDataRead(KSyncML::SyncMLDatabase *);
	void databaseDataWritten(KSyncML::SyncMLDatabase *);
	void sessionFinished();
	
public slots:
	//XXX FIXME change the name of that:
	void slotParsingFinished();
	void slotEverythingFinished();
	void slotEverythingDisconnected();

	void slotDataReceived(SyncMLSession::TSynceesList& data, SMLURI& database);
	void slotKeysReceived(SyncMLSession::TKeysList& data, SMLURI& database);

public:
	SMLSessionHandler(MLHttpServer * communication_, SMLVersion protocolVersion = UNKNOWN);
	virtual ~SMLSessionHandler();

	void writeDatabases();
	void readDatabases();

	void forceSend();
  
  SMLVersion getVer() { return syncmlVersion; }
  //changing the SyncML version during the session 
  //is not supported
  void setVer(SMLVersion newVer);
  
  SMLURI* getURI() { return &URI; }
  void setURI(SMLURI u) { URI=u; }

  SMLURI* getRemoteURI() { return &remoteURI; }
  void setRemoteURI(SMLURI u) { remoteURI=u; }

  // Help functions
  SMLResponsePacket* getResponse() { return response; }
  SMLDevInf* getDevInf(); 
  
  //starting from now, session handler has only one local devinf void setDevInf(SMLDevInf *d) { DevInf=d; }

  SMLDevInf* getRemoteDevInf() { return remoteDevInf; }
  void setRemoteDevInf(SMLDevInf *d) { 
    if (remoteDevInf!=NULL) delete remoteDevInf;
    remoteDevInf = d;
  }

  SMLCmdMapType* getPending() { return &pending; }
  KSyncML::SyncMLDatabaseMapType* getDBMap() { return &databaseMap; };
  
	void addDatabase(KSyncML::SyncMLDatabase *data);
	bool hasDatabase(const SMLURI & uri);
  
  
  SMLErrorType getResult(SMLCmdRef ref);

  
  void sendDatabaseChangeLog(KSyncML::SyncMLDatabase *);

	virtual SMLCmdRef sendTwoWaySync(SMLURI source, SMLURI target, string next=""); 
	virtual SMLCmdRef sendOneWaySync(SMLURI source, SMLURI target, string next=""); 

  
  virtual SMLCmdRef sendDevInf();
  virtual SMLCmdRef requestDevInf();

  //XXX TODO: Use this functions
//  virtual SMLCmdRef sendAdd(SMLTreeNode *syncNode, SMLType *type, SMLURI *source, SMLURI *target, string data);
//  virtual SMLCmdRef sendCopy(SMLTreeNode *syncNode, SMLType *type, SMLURI *source, SMLURI *target);
//  virtual SMLCmdRef sendDelete(SMLTreeNode *syncNode, SMLType *type, SMLURI *source, SMLURI *target=0);
//  virtual SMLCmdRef sendReplace(SMLTreeNode *syncNode, SMLType *type, SMLURI *source, SMLURI *target, string data);

  // functions for receiving
  virtual void recDisplay(string what);
  virtual void recDevInf();
};


#endif

