#include "mlsynchistory.h"

#include <kconfig.h>
#include <kdebug.h>

namespace KSyncML {

SyncHistoryMap::SyncHistoryMap( const QString& file )
	: mFile( file ), mConf( 0l )
	{}

SyncHistoryMap::~SyncHistoryMap()
{
	delete mConf;
}

QString SyncHistoryMap::fileName()const
{
	return mFile;
}

void SyncHistoryMap::setFileName( const QString& fileName )
{
  mFile = fileName;
}


void SyncHistoryMap::load()
{
	if ( mFile.isEmpty() )
		return;

	KConfig* conf = config();
	QStringList list = conf->groupList();

	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it ) 
	{
		conf->setGroup( (*it) );
		
		HistoryData data;
		data.localUID = conf->readEntry("localUID");
		data.remoteUID = conf->readEntry("remoteUID");
		data.md5 = conf->readEntry("checksum");
		//kdDebug() << "MLSyncHistoryMap: loaded '" << (*it) << "' (l=" << data.localUID << ", r=" << data.remoteUID << ")" << endl;
		
		historyMap.insert((*it), data);
	}
	kdDebug() << "*********** SyncHistoryMap::load data count = " << historyMap.count() << endl;
}


/**
 * Saves the current Map to a permanent storage and
 * wipes out any previous data.
 * Calling without a prior call to load() works as well
 * as the Config object is created on demand
 */
void SyncHistoryMap::save()
{
	kdDebug() << "*********** SyncHistoryMap::save data count = " << historyMap.count() << endl;
	KConfig* conf = config();

	QStringList groups = conf->groupList();
	for (QStringList::Iterator it = groups.begin(); it != groups.end(); ++it )
		conf->deleteGroup( (*it) );
	
	QString key;
	Iterator it;
	for ( it = historyMap.begin(); it != historyMap.end(); ++it ) 
	{
		conf->setGroup( it.key() );
		HistoryData data = it.data();
		
		conf->writeEntry( "localUID", data.localUID );
		conf->writeEntry( "remoteUID", data.remoteUID );
		conf->writeEntry( "checksum", data.md5 );
		
		kdDebug() << "MLSyncHistoryMap: saved '" << it.key() << "' (l=" << data.localUID << ", r=" << data.remoteUID << ")" << endl;
	}
	
	conf->sync();
}



/*
QString SyncHistoryMap::text( const QString& id_key )const
{
	return mMap[id_key];
}

QString SyncHistoryMap::localKey( const QString& id_key )const
{
	return mKeyMap[id_key];
}


QString SyncHistoryMap::remoteKey( const QString & localKey)const
{
	for (ConstIterator  it = mKeyMap.begin(); it != mKeyMap.end(); ++it ) 
	{
		if (it.data() == localKey)
		{
			return it.key();
		}
	}
	return "";
}
*/

/**
 * @return True if the @param id_key is inside the internal map
 */
bool SyncHistoryMap::contains( const QString& id_key )const
{
  	return historyMap.contains( id_key );
	//return mMap.contains( id_key );
}

QString SyncHistoryMap::localUID(const QString& id_key ) const
{
	if (historyMap.contains(id_key))
		return historyMap[id_key].localUID;
	else
		return "";
}

QString SyncHistoryMap::remoteUID(const QString& id_key ) const
{
	if (historyMap.contains(id_key))
		return historyMap[id_key].remoteUID;
	else
		return "";
}

QString SyncHistoryMap::md5(const QString& id_key ) const
{
	if (historyMap.contains(id_key))
		return historyMap[id_key].md5;
	else
		return "";
}

void SyncHistoryMap::setLocalUID( const QString& id_key, const QString& localKey )
{
  HistoryData data = historyMap[id_key];
  data.localUID = localKey;
  historyMap[id_key] = data;
}

void SyncHistoryMap::setRemoteUID( const QString& id_key, const QString& remoteKey )
{
  HistoryData data = historyMap[id_key];
  data.remoteUID = remoteKey;
  historyMap[id_key] = data;
}

void SyncHistoryMap::setMD5( const QString& id_key, const QString& text_data )
{
  HistoryData data = historyMap[id_key];
  data.md5 = text_data;
  historyMap[id_key] = data;
}

void SyncHistoryMap::move(const QString & old_key, const QString & new_key)
{
  HistoryData data = historyMap[old_key];
  data.remoteUID = new_key;

  historyMap.remove(old_key);
  historyMap.insert(new_key, data);
}


/**
 * Clears the permanent storage and also
 * the internal map
 */
void SyncHistoryMap::clear()
{
	historyMap.clear();
	KConfig* conf = config();
	QStringList groups = conf->groupList();
	for (QStringList::Iterator it = groups.begin(); it != groups.end(); ++it ) {
		conf->deleteGroup( (*it) );
	}
}

/**
 * creates the kconfig object on demand
 */
KConfig* SyncHistoryMap::config()
{
	if (!mConf )
		mConf = new KConfig( mFile, false, false );

	return mConf;
}


}
