#include <qdir.h>
#include <kdebug.h>

#include "syncml.h"
#include "syncmldatabase.h"

namespace KSyncML
{

ChangeLog::~ChangeLog()
{
	QValueList<Record *>::iterator it;

    for ( it = records.begin(); it != records.end(); ++it )
		delete (*it);
}

QString ChangeLog::actionToString(TAction action)
{
	switch (action) {
		case OK: return "OK"; case ADD: return "ADD"; case REPLACE: return "REPLACE"; case DELETE: return "DELETE"; default: return "UNKNOWN";
	}
}

void ChangeLog::clear()
{
	QValueList<Record *>::iterator it;

    for ( it = records.begin(); it != records.end(); ++it )
		delete (*it);
	
	records.clear();
}

void ChangeLog::dump()
{
	QValueList<Record *>::iterator it;

    for ( it = records.begin(); it != records.end(); ++it )
	{
		kdDebug() << "ChangeLog: record " << (*it)->item->ID << " action " << actionToString((*it)->action) << endl;
	}
}




SyncMLDatabase::SyncMLDatabase(const QString & databaseName): URI(databaseName), _gotData(false), syncee(NULL)
{
	//changeLog = new ChangeLog();
}

SyncMLDatabase::~SyncMLDatabase() 
{
	delete syncee;
}

void SyncMLDatabase::setRemoteURI(const QString & remoteID)
{
	remoteURI = SMLURI(remoteID);
}

void SyncMLDatabase::initializeDatabase(const QString & remoteDeviceID)
{
	setRemoteURI(remoteDeviceID);
	
	QString m_remoteDeviceID = getRemoteDatabaseID();
	
	m_databaseDir = QDir::homeDirPath() + "/.kitchensync/KSyncML/" + m_remoteDeviceID + "/" + getDatabaseID();
	QDir di(m_databaseDir );
	
	if ( di.exists() ) 
	{
		//firstsync = false;
		kdDebug() << "SyncMLDatabase: database dir already existing." << endl;
	} 
	else 
	{
		//firstsync = true;
		kdDebug() << "SyncMLDatabase: Creating dirs: " << (m_databaseDir) << endl;
		QDir dir;
		dir.mkdir(QDir::homeDirPath() + "/.kitchensync/");
		dir.mkdir(QDir::homeDirPath() + "/.kitchensync/KSyncML/");
		dir.mkdir(QDir::homeDirPath() + "/.kitchensync/KSyncML/" + m_remoteDeviceID);
		dir.mkdir(QDir::homeDirPath() + "/.kitchensync/KSyncML/" + m_remoteDeviceID + "/" + getDatabaseID());
	}
}


} //end of namespace
#include "syncmldatabase.moc"





