#ifndef __SYNCML_DATABASE_H
#define __SYNCML_DATABASE_H

#include <qstring.h>
#include <qmap.h>


#include "syncml.h"
#include "mlsession.h"

#include "syncee.h"
#include "syncmluri.h"
//#include "mlsynchistory.h"

class SMLDevInf;
class SMLSessionHandler;
class SyncMLItem;

namespace KSyncML 
{

class ChangeLog
{
public:
	typedef enum {OK, ADD, REPLACE, DELETE} TAction;

	class Record
	{
	public:
		Record(TAction _action, SyncMLItem * _item): action(_action), item(_item)
		{
		}
	//protected:
		TAction action;
		SyncMLItem *item;
		
		//friend class Changelog;

	};


	ChangeLog() {};
	~ChangeLog();

	void clear();
	
	QValueList<Record *> &recordsList() { return records ;};

	void dump();
	QString actionToString(TAction action);
	
	//don't delete the r pointer after calling addRecord method
	void addRecord(Record * r) {records.append(r);}; 
	
	QValueList<Record *> records;
};


/**
// This class is a mix of the original LibSyncML code, my hacking
// and the ideas from the irmcsync konnector. Don't be too scared.

// The SyncMLDatabase is a class which stores all the data
// from one database of the given client together with some parameters
// The algorithm would be as follows :
// 1. Read the local data
// 2. Read the changes from the client
// 3. Update local data
// 4. Emit synceesRead, and wait
// 5. On writeSyncees check the differences between the current
//    data and the updated one from the point 3.
// 6. Push the changes to the client
// 7. Update once again the local data
// 8. Commit.

// For now, nothing of it is implemented. We will start
// by handling correctly 3-6 and then the rest.
*/
class SyncMLDatabase: public QObject 
{
Q_OBJECT

public:
	typedef std::map<std::string,bool> SyncCapType;
  
	struct DataStoreType
	{
		std::string SourceRef;
		std::string DisplayName;
		SMLUInt MaxGUIDSize;
		
		bool SharedMem;
		// FIXME: 4 GB are NOT enough for everybody...
		// but unfortunately uint64 is not ansi :(
		// Maciek: what is this about?
		SMLUInt MaxMem;
		SMLUInt MaxID;
		SyncCapType SyncCap;
		
		std::pair<std::string,std::string> RxPref;
		std::multimap<std::string,std::string> Rx;
		std::pair<std::string,std::string> TxPref;
		std::multimap<std::string,std::string> Tx;
		
		DataStoreType() : SourceRef(""), DisplayName(""), MaxGUIDSize(0),SharedMem(false),
			MaxMem(0), MaxID(0), SyncCap(), RxPref(), Rx(), TxPref(), Tx()  {  }
	};
  

	SyncMLDatabase(const QString & databaseName);
	virtual ~SyncMLDatabase();
	
	void initializeDatabase(const QString & remoteDeviceID);
	
	//SMLDevInf *getDevInf() { return devInf; }
	SMLURI getURI () { return URI; }
	
	DataStoreType * getDataStoreInfo() { return &dataStore; }
	
	KSync::Syncee * getSyncee() { return syncee; }

	bool shouldBeSynchronized() { return _gotData; };
	
	virtual void processClientData(SyncMLSession::TSynceesList& data) = 0;
	virtual void processClientKeys(SyncMLSession::TKeysList& data) = 0;

public slots:
	//
	// it's the responsibility of the client to delete the pointer.
	virtual ChangeLog * getDBChangeLog() = 0;
	
signals:
	void synceesRead(KSyncML::SyncMLDatabase *);
	void synceesWritten(KSyncML::SyncMLDatabase *);
	void syncFinished(KSyncML::SyncMLDatabase *);
	
protected:
	//SMLDevInf *devInf;
	SMLURI URI;
	SMLURI remoteURI;
	void setRemoteURI(const QString & remoteID);
	
	DataStoreType dataStore;
	
	
	/** If true, the database has got some data from the client and
	* should be synchronized.
	*/
	bool _gotData;
	
	/** The official repository for the syncee data (the one that
	 really get synced with the other konnectors)
	*/
	KSync::Syncee *syncee;
		
	/** The local copy of the other side items.
	 */
	QValueList<SyncMLItem *> items;

	/** It can be used in order to make a directory on the hard disk.
	*/
	QString getDatabaseID() const
	{
		return URI.getQLocURI(false);
	}
	
	/** It can be used in order to make a directory on the hard disk.
	 */
	QString getRemoteDatabaseID() const
	{
		return remoteURI.getQLocURI(false);
	}
	
	/** The changelog to send to the other side.
	*/
	//////ChangeLog * changeLog;
	
	/** Local directory with data of the database
	*/
	QString m_databaseDir;

	/** Temporary dictionnary of mapping 
	* tempGUID => KDE keys
	*/
	QMap<QString, QString> remoteURI_UID_Dict;

}; // end of class

typedef QMap<QString, SyncMLDatabase*> SyncMLDatabaseMapType;

}; //end of namespace

#endif


