/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#include <sys/types.h>
#include "syncml.h"
#include "syncmluri.h"
#include "xmlparser_tree.h"

void SMLURI::addToNode(SMLTreeNode *t, bool addSlash) {
  // Adding the nodes and removing them if empty is not efficient, but
  // it's much clearer to read than a lot of if's
  t->eliminateChildren();
  new SMLTreeNode(t,SMLToken::LocURI, QString(getLocURI(addSlash).c_str()));
  new SMLTreeNode(t,SMLToken::LocName,QString(getLocName().c_str()));
  t->eliminateEmptyChildren();

}
void SMLURI::addDirectlyToNode(SMLTreeNode *t, bool addSlash)
{
	t->setSTLValue((getLocURI(addSlash)));
}

SMLURI::SMLURI(SMLTreeNode *t, SMLURI *u) {
  setBase(u);
  setLocURI(QString("")); 
  setLocName(QString(""));
  if (t!=NULL) {
    SMLTreeNode *n;
    if ((n=t->getChildOfType(SMLToken::LocURI))!=NULL) setLocURI(n->getSTLValue()); 
    if ((n=t->getChildOfType(SMLToken::LocName))!=NULL) setLocName(n->getSTLValue()); 
  }
}

string SMLURI::getLocURI(bool addSlash) const
{ 
  //XXX TODO why that??
	if (base && addSlash) return "./"+LocURI;
  
  
  return LocURI; 
}

QString SMLURI::getQLocURI(bool addSlash) const
{ 
	return QString(getLocURI(addSlash).c_str()); 
}

void SMLURI::setLocURI(const QString & qs)
{
	string s(qs.ascii());
	//setLocURI(s.ascii());
	
	if (base) 
	{
		if (s.find(base->getAbsoluteLocURI())==0) 
		{
			s = s.substr(base->getAbsoluteLocURI().length()+1);
		}
	}

	if (!s.empty()) {
		if ((s.substr(0,2))=="./") s = s.substr(2);
    
		if (s.substr(0,1)=="/") {
			if (base) {
				string bStr = base->getAbsoluteLocURI();
				string tStr = "";
				bool lastslash=false;
				for (string::iterator i=bStr.begin();i!=bStr.end();i++) {
					if (lastslash) {
						if ((*i)=='/') lastslash=false; else {
							tStr = bStr.replace(--i,bStr.end(),s);
							i = bStr.end();
							i--;
						}
					} else {
						if ((*i)=='/') lastslash=true;
					}
				}
				if (tStr=="") {
					s = s.substr(1);
				} else {
					base=0;
					s = tStr;
				}
			} else {
				s = s.substr(1);
			}
		}
	}

	if (!s.empty()) {
		if (s[s.length()-1]=='/') s = s.substr(0,s.length()-1);
	}

	LocURI=s; 

	
}
void SMLURI::setLocURI(string s) { 

  if (base) {
    if (s.find(base->getAbsoluteLocURI())==0) {
      s = s.substr(base->getAbsoluteLocURI().length()+1);
    }
  }

  if (!s.empty()) {
    if ((s.substr(0,2))=="./") s = s.substr(2);
    
    if (s.substr(0,1)=="/") {
      if (base) {
	string bStr = base->getAbsoluteLocURI();
	string tStr = "";
	bool lastslash=false;
	for (string::iterator i=bStr.begin();i!=bStr.end();i++) {
	  if (lastslash) {
	    if ((*i)=='/') lastslash=false; else {
	      tStr = bStr.replace(--i,bStr.end(),s);
	      i = bStr.end();
	      i--;
	    }
	  } else {
	    if ((*i)=='/') lastslash=true;
	  }
	}
	if (tStr=="") {
	  s = s.substr(1);
	} else {
	  base=0;
	  s = tStr;
	}
      } else {
	s = s.substr(1);
      }
    }
  }

  if (!s.empty()) {
    if (s[s.length()-1]=='/') s = s.substr(0,s.length()-1);
  }

  LocURI=s; 
}

QString SMLURI::getQAbsoluteLocURI() const 
{ 
	return QString(getAbsoluteLocURI().c_str()); 
}


string SMLURI::getAbsoluteLocURI() const
{ 
  if (base) return base->getAbsoluteLocURI()+"/"+LocURI;
  return LocURI; 
}

void SMLURI::setBase(SMLURI *u) { 
  base=u; 
}

void SMLURI::addBase(SMLURI *u) { 
  string oldURI=getAbsoluteLocURI();
  setBase(u); 
  setLocURI(oldURI);
}

