/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#ifndef __SMLFLATTENER_H
#define __SMLFLATTENER_H

#include <qtextstream.h>

class SMLFlattener {
 protected:
  QTextStream *outputStream;
  SMLNamespaceContainer *namespaceContainer;
 public:
  SMLFlattener(QTextStream *o, SMLNamespaceContainer *ns) { 
    outputStream = o;
    namespaceContainer = ns;
  }
  virtual bool header() = 0;
  virtual bool startTag(SMLTokenType token, SMLTokenSpaceType parent, string *s) = 0;
  virtual bool endTag(SMLTokenType token, string *s) = 0;
  virtual bool emptyTag(SMLTokenType token, SMLTokenSpaceType parent, string *s) {
    startTag(token,parent,s);
    endTag(token,s);
    return true;
  }
  virtual bool text(string *s) = 0;
  void flush() {
    outputStream->device()->flush();
  }
};

class SyncMLFlattener:public SMLFlattener {
 protected:
  virtual string addNamespace(SMLTokenSpaceType mySpace, SMLTokenSpaceType parent);
  virtual string buildStartTag(SMLTokenType token, SMLTokenSpaceType parent, string *s); 
 public:
  SyncMLFlattener(QTextStream *o, SMLNamespaceContainer *ns) : SMLFlattener(o,ns) {};
  virtual bool header();
  virtual bool startTag(SMLTokenType token, SMLTokenSpaceType parent, string *s);
  virtual bool endTag(SMLTokenType token, string *s);
  virtual bool text(string *s);
};

class NiceSyncMLFlattener:public SyncMLFlattener {
 protected:
  SMLUInt indentCount;
  SMLUInt indentBy;
  enum last_enum {open,txt,close} last;
 public:
  NiceSyncMLFlattener(QTextStream *o, SMLNamespaceContainer *ns, SMLUInt indent=2);
  virtual bool startTag(SMLTokenType token, SMLTokenSpaceType parent, string *s);
  virtual bool endTag(SMLTokenType token, string *s);
  virtual bool emptyTag(SMLTokenType token, SMLTokenSpaceType parent, string *s);
  virtual bool text(string *s);
};

#endif
