/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#include "syncml.h"
#include "xmlnamespace.h"

#include <kdebug.h>

#define NAMESPACE_DEBUG

SMLNamespaceContainer::SMLNamespaceContainer(SMLVersion syncmlVersion) 
{
	#ifdef DEBUG_MEM
	kdDebug() << "DEBUG_MEM - SMLNamespaceContainer: created." << endl;
	#endif
	
	nextNamespace=SMLNamespace::OtherStart;

	//addNewNamespace("SYNCML:SYNCML1.0",SMLNamespace::SyncML);
	if (syncmlVersion == ONE_ZERO)
		addNewNamespace("SYNCML:SYNCML1.0",SMLNamespace::SyncML);
	else if (syncmlVersion == ONE_ONE)
		addNewNamespace("SYNCML:SYNCML1.1",SMLNamespace::SyncML);
    
	
	//addNewNamespace("syncml:syncml",SMLNamespace::SyncML);
	addNewNamespace("syncml:metinf",SMLNamespace::MetInf);
	addNewNamespace("syncml:devinf",SMLNamespace::DevInf);

	//are these useful?
	//addNewNamespace("http://www.syncml.org/docs/syncml_represent_v10_2001207.dtd",SMLNamespace::SyncML);
	//addNewNamespace("http://www.syncml.org/docs/metinf_v101_20010530.dtd",SMLNamespace::MetInf);
	//addNewNamespace("http://www.syncml.org/docs/devinf_v10_20001207.dtd",SMLNamespace::DevInf);
}

SMLNamespaceContainer::~SMLNamespaceContainer()
{
	#ifdef DEBUG_MEM
	kdDebug() << "DEBUG_MEM - SMLNamespaceContainer: destroyed." << endl;
	#endif
}

string SMLNamespaceContainer::getSyncMLStringForNamespace(SMLTokenSpaceType space) {
  return namespaceName[space];
}

SMLTokenSpaceType SMLNamespaceContainer::addNewNamespace(QString str, SMLTokenSpaceType asNr) 
{
  
	string s(str.ascii());
	if (namespaceCode[s] != 0) 
	{
		//#ifdef NAMESPACE_DEBUG
		//kdDebug() << "SMLNamespaceContainer : returning code " << namespaceCode[s].c_str() << " for namespace: " << str << endl;
		//#endif
	  	return namespaceCode[s];
	}
  
	#ifdef NAMESPACE_DEBUG
	kdDebug() << "SMLNamespaceContainer : new namespace: " << str << endl;
	#endif
  
	
	
	if (asNr==0) 
	{
		//XXX FIXME well, we should really invent a way to correctly detect
		//all 1.0/1.1 namespaces. What are they, by the way?
		if (str.compare("SYNCML:SYNCML1.0") == 0)
			return addNewNamespace("SYNCML:SYNCML1.0", SMLNamespace::SyncML);	
		else if (str.compare("SYNCML:SYNCML1.1") == 0)
			return addNewNamespace("SYNCML:SYNCML1.1", SMLNamespace::SyncML);
		else
		{
			asNr = nextNamespace;
			// FIXME
			// approx. 200 Alien Namespaces should be enough for everybody!!!! (at least in 1 session)
			// This is the main reason this is a class and gets rebuild every time,
			// although that is not as efficient. but otherwhise a long-running server
			// might actually encounter more than 200 alien namespaces.
			if (asNr>SMLNamespace::OtherEnd) 
				asNr=SMLNamespace::OtherEnd; 
			nextNamespace++;
		}
	}
	namespaceCode[s] = asNr;
	if (namespaceName[asNr].length() == 0) 
		namespaceName[asNr] = s;
	return asNr;

}


