/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#ifndef __PARSER_TREE_H
#define __PARSER_TREE_H

#include <qstring.h>

#include <list>
#include "syncml.h"

class SMLTreeNode {
public:
 typedef list<SMLTreeNode*> nodeList;
 private:
  nodeList children;

  SMLTreeNode *parent;
  SMLTokenType myType;
  
  //we try to break as little things as possible
  QString value;
 protected:
 public:
  SMLTreeNode(SMLTreeNode* parentNode, SMLTokenType
              type, std::string val);
  SMLTreeNode(SMLTreeNode* parentNode, SMLTokenType
              type, QString val="");
  SMLTreeNode(SMLTreeNode* copyme);
  nodeList* getChildren();
  SMLTreeNode* getParent();
  
  QString getValue() { return value; };
  void setValue(const QString & v) { value = v; };
  
  //you will loose the unicode by getting the STLValue!
  string getSTLValue();
  string getSimplifiedSTLValue();
  void setSTLValue(string value);
  SMLTokenType getType ();
  bool simplifyMe();
  bool hasChildOfType(SMLTokenType type);
  bool flatten(SMLFlattener *flat);
  void addChild(SMLTreeNode *newChild);
  void orderChildren(SMLUInt ocount,SMLTokenType *order);
  void mustHaveChildren(SMLUInt ocount,SMLTokenType *children);
  SMLTreeNode* getChildOfType(SMLUInt ocount, SMLTokenType *path);
  SMLTreeNode* getChildOfType(SMLTokenType which) { return
                                                      getChildOfType(1,&which);}
  bool isEmpty();
  void eliminateTextChildren();
  void eliminateEmptyChildren();
  void eliminateChildren();
  ~SMLTreeNode();
  
  //pseudo dump
  void dump(int indentation = 0);
  
};



#endif
